package com.flossom.system.service;

import com.flossom.common.core.domain.entity.SysDept;
import com.flossom.common.core.domain.entity.SysTag;
import com.flossom.common.core.domain.vo.TreeSelect;

import java.util.List;
import java.util.Map;


/**
 * 标签管理 服务层
 *
 * @author flossom
 */
public interface ISysTagService
{
    /**
     * 查询标签管理数据
     *
     * @param tag 标签信息
     * @return 标签信息集合
     */
    public List<SysTag> selectDeptList(SysTag tag);

    /**
     * 查询标签树结构信息
     *
     * @param tag 标签信息
     * @return 标签树信息集合
     */
    public List<TreeSelect> selectDeptTreeList(SysTag tag);

    /**
     * 构建前端所需要树结构
     *
     * @param tags 标签列表
     * @return 树结构列表
     */
    public List<SysTag> buildDeptTree(List<SysTag> tags);

    /**
     * 构建前端所需要下拉树结构
     *
     * @param tags 标签列表
     * @return 下拉树结构列表
     */
    public List<TreeSelect> buildDeptTreeSelect(List<SysTag> tags);

    /**
     * 根据角色ID查询标签树信息
     *
     * @param roleId 角色ID
     * @return 选中标签列表
     */
    public List<Long> selectDeptListByRoleId(Long roleId);

    /**
     * 根据标签ID查询信息
     *
     * @param id 标签ID
     * @return 标签信息
     */
    public SysTag selectDeptById(Long id);

    /**
     * 根据ID查询所有子标签（正常状态）
     *
     * @param id 标签ID
     * @return 子标签数
     */
    public int selectNormalChildrenDeptById(Long id);

    /**
     * 是否存在标签子节点
     *
     * @param id 标签ID
     * @return 结果
     */
    public boolean hasChildByDeptId(Long id);

    /**
     * 查询标签是否存在用户
     *
     * @param id 标签ID
     * @return 结果 true 存在 false 不存在
     */
    public boolean checkDeptExistUser(Long id);

    /**
     * 校验标签名称是否唯一
     *
     * @param tag 标签信息
     * @return 结果
     */
    public boolean checkDeptNameUnique(SysTag tag);

    /**
     * 校验标签是否有数据权限
     *
     * @param id 标签id
     */
    public void checkDeptDataScope(Long id);

    /**
     * 新增保存标签信息
     *
     * @param tag 标签信息
     * @return 结果
     */
    public int insertDept(SysTag tag);

    /**
     * 修改保存标签信息
     *
     * @param tag 标签信息
     * @return 结果
     */
    public int updateDept(SysTag tag);

    /**
     * 删除标签管理信息
     *
     * @param id 标签ID
     * @return 结果
     */
    public int deleteDeptById(Long id);

    /**
     * 查询部门树结构信息
     *
     * @param tag 标签信息
     * @return 标签树信息集合
     */
    public List<TreeSelect> selectTagTreeList(SysTag tag);

    List<Map<String, Object>> getMiniProgramTags();

    List<Map<String, Object>> getWecomTags();
}
